"""
Publisher for swarm-bus
"""
import datetime
import decimal
import json
import logging

logger = logging.getLogger(__name__)


class ComplexEncoder(json.JSONEncoder):

    def default(self, obj):
        if isinstance(obj, datetime.datetime):
            return obj.isoformat()
        if isinstance(obj, datetime.date):
            return obj.isoformat()
        if isinstance(obj, datetime.timedelta):
            return obj.total_seconds()
        if isinstance(obj, decimal.Decimal):
            return float(obj)

        return super(ComplexEncoder, self).default(obj)


def json_dumps(datas):
    """
    JSON Pretty dumping
    """
    return json.dumps(
        datas,
        cls=ComplexEncoder,
        indent=4,
        sort_keys=True
    )


class Publisher(object):
    """
    Publisher logic
    """

    def publish(self, queue_name, datas={}, priority=0,
                delay=0, attributes={}):
        """
        Publish message datas into queue
        """
        attributes = attributes.copy()
        queue = self.get_queue(queue_name, priority)

        logger.info(
            "[%s] Message published on '%s'",
            self.log_namespace,
            queue.url
        )

        queue.send_message(
            MessageBody=json_dumps(datas),
            MessageAttributes=attributes,
            DelaySeconds=delay
        )



"""
Connector for swarm-bus
"""
import logging
import os
import socket

from swarm_bus.constants import OFFICE_HOURS
from swarm_bus.constants import POLLING_INTERVAL
from swarm_bus.constants import PREFIX
from swarm_bus.constants import PRIORITIES
from swarm_bus.constants import REGION
from swarm_bus.constants import RETENTION_PERIOD
from swarm_bus.constants import SLEEP_TIME
from swarm_bus.constants import VISIBILITY_TIMEOUT


logger = logging.getLogger(__name__)


class Transport(object):
    """
    Default transport configuration for SQS
    """
    def __init__(self,
                 access_key,
                 secret_key,
                 prefix=PREFIX,
                 priorities=PRIORITIES,
                 polling_interval=POLLING_INTERVAL,
                 retention_period=RETENTION_PERIOD,
                 visibility_timeout=VISIBILITY_TIMEOUT,
                 sleep_time=SLEEP_TIME,
                 office_hours=OFFICE_HOURS,
                 region=REGION):
        # S3
        self.access_key = access_key
        self.secret_key = secret_key

        # SQS
        self.region = region
        self.prefix = prefix % {
            'hostname': self.get_hostname()
        }

        # Priorities
        self.priorities = priorities

        # Offfice hours
        self.office_hours = office_hours

        # Queue default config
        self.polling_interval = polling_interval
        self.retention_period = retention_period
        self.visibility_timeout = visibility_timeout
        self.sleep_time = sleep_time

    def get_hostname(self):
        """
        Method for having consistent hostname accross hosts.
        """
        default = socket.gethostname()
        hostname = os.getenv('AMQP_HOSTNAME', default)

        return hostname



"""
Deleter of swarm-bus
"""
import logging

logger = logging.getLogger(__name__)


class Deleter(object):
    """
    Deleter logic
    """

    def delete_queue(self, queue_name):
        """
        Delete queue
        """
        queue_set = self.get_queue_set(queue_name)

        for queue_priority in queue_set.queues:
            queue_priority.delete()

            logger.info(
                "[%s] Queue '%s' is now deleted",
                self.log_namespace,
                queue_priority.url
            )



"""
Consumer for swarm-bus
"""
import datetime
import json
import logging
import time

logger = logging.getLogger(__name__)


class Consumer(object):
    """
    Consumer logic
    """

    def consume(self, queue_name, callback=None, error_handler=None,
                messages=10):
        """
        Consume message datas
        """
        try:
            queue_set = self.get_queue_set(queue_name)
        except KeyError:
            raise ValueError(
                "'%s' is an unknown queue" % queue_name
            )

        if callback is None:
            raise ValueError(
                "callback parameter can not be empty"
            )

        callback_wrapped = self.callback_wrapper(callback, error_handler)

        while True:
            if not self.can_consume:
                logger.debug(
                    '[%s] Consuming is on hold. Next try in 60 seconds',
                    self.log_namespace
                )
                sleep_time = 60
            else:
                for queue_priority in queue_set.queues:
                    logger.debug(
                        '[%s] Polling %s for %s seconds',
                        self.log_namespace,
                        queue_priority.url,
                        queue_set.polling_interval
                    )
                    polled_messages = queue_priority.receive_messages(
                        WaitTimeSeconds=queue_set.polling_interval,
                        MaxNumberOfMessages=messages
                    )
                    logger.debug(
                        '[%s] %s messages polled',
                        self.log_namespace,
                        len(polled_messages)
                    )
                    for message in polled_messages:
                        callback_wrapped(
                            json.loads(message.body),
                            message
                        )

                sleep_time = queue_set.sleep_time
                if sleep_time:
                    logger.debug(
                        '[%s] Sleeping for %s seconds',
                        self.log_namespace,
                        sleep_time
                    )

            time.sleep(sleep_time)

    @property
    def can_consume(self):
        """
        Check a queue can be consumed.
        """
        if not self.transport.office_hours:
            return True

        now = datetime.datetime.now()
        if now.weekday() in [5, 6]:  # Week-end
            return False

        hour = now.hour
        if hour >= 9 and hour < 20:
            return True

        return False

    def callback_wrapper(self, callback, error_handler):
        """
        Decorate the callback to log exceptions
        and send them to Senty later if possible.

        Also cancels the exception to avoid process to crash !
        """
        def exception_catcher(body, message):
            """
            Decorator around callback.
            """
            try:
                return callback(body, message)
            except Exception:
                logger.exception(
                    '[%s] Unhandled exception occured !',
                    self.log_namespace
                )
                if error_handler:
                    error_handler(body, message)
                    logger.debug(
                        '[%s] Error handler called',
                        self.log_namespace
                    )

        return exception_catcher



"""
Swarm bus
"""
from setuptools import find_packages
from setuptools import setup

__version__ = '5.1'
__license__ = 'GPL License'

__author__ = 'The Swarm Team'
__email__ = 'dev@hiventy.com'

__url__ = 'https://bitbucket.org/monalgroup/swarm-bus'


install_requires = [
    'boto3==1.14.20',
]


setup(
    name='swarm-bus',
    version=__version__,
    license=__license__,

    description='ESB SQS based',
    long_description=open('README.rst').read(),
    keywords='ESB, tools, swarm, bus',

    packages=find_packages(exclude=('tests', 'tests.*')),

    author=__author__,
    author_email=__email__,
    url=__url__,

    classifiers=[
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Topic :: Software Development',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
    ],

    install_requires=install_requires,

    zip_safe=False,
    include_package_data=True,
)
